package speiger.src.builder.mappers;

import java.util.regex.Pattern;

import speiger.src.builder.misc.RegexMatcher;
import speiger.src.builder.misc.RegexUtil;

public class ArgumentMapper implements IMapper
{
	String searchValue;
	Pattern pattern;
	String replacement;
	String argumentBreaker;
	String braces = "()";
	boolean removeBraces;
	
	public ArgumentMapper(String pattern, String replacement, String argumentBreaker)
	{
		this(pattern, pattern, replacement, argumentBreaker);
	}
	
	public ArgumentMapper(String searchValue, String pattern, String replacement, String argumentBreaker)
	{
		this.searchValue = searchValue;
		this.pattern = Pattern.compile(pattern);
		this.replacement = replacement;
		this.argumentBreaker = argumentBreaker;
	}
	
	public ArgumentMapper setBraceType(String s) 
	{
		if(s.length() != 2) throw new IllegalStateException("Start and End char only");
		braces = s;
		return this;
	}
	
	public ArgumentMapper removeBraces()
	{
		removeBraces = true;
		return this;
	}
	
	@Override
	public String getSearchValue()
	{
		return searchValue;
	}
	
	@Override
	public String apply(String t)
	{
		RegexMatcher matcher = new RegexMatcher(pattern, t);
		try
		{
			if(matcher.find())
			{
				StringBuffer buffer = new StringBuffer();
				do
				{
					String text = RegexUtil.searchUntil(t, matcher.end()-1, braces.charAt(0), braces.charAt(1));
					if(!text.isEmpty())
					{
						matcher.appendReplacement(buffer, "").skip(text.length());
						buffer.append(String.format(replacement, (Object[])getString(text).split(argumentBreaker)));
					}
				}
				while(matcher.find());
				matcher.appendTail(buffer);
				return apply(buffer.toString());
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		catch(Error error) 
		{
			System.out.println("Error with ["+pattern.pattern()+"] pattern");
			throw error;
		}
		return t;
	}
	
	protected String getString(String s)
	{
		return removeBraces ? s.substring(1, s.length() - 1) : s;
	}
}
